// src/backend/dynamic_thread.cc - Copyright 2005, 2006, University
//               of Padova, dept. of Pure and Applied
//               Mathematics
//
// This file is part of SGPEMv2.
//
// This is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// SGPEMv2 is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with SGPEMv2; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

#include "dynamic_thread.hh"
#include "static_thread.hh"
#include "dynamic_request.hh"
#include <cassert>

#include "smartp.tcc"

using namespace sgpem;
using std::vector;


DynamicThread::DynamicThread(StaticThread* core, DynamicProcess* parent) : 
  DynamicSchedulable(*core), _state(state_future), _parent(parent)
{
}
 
DynamicThread::DynamicThread(const DynamicThread &other) :
  Schedulable(), DynamicSchedulable(other), Thread()
{
  typedef vector<DynamicRequest*>::const_iterator ReqIt;
  
  const vector<DynamicRequest*>& other_req = other._dynamic_requests;
  
  _state = other._state;
  _parent = other._parent;

  for(ReqIt it = other_req.begin(); it != other_req.end(); ++it)
    _dynamic_requests.push_back(new DynamicRequest(*(*it)));
}

DynamicProcess& 
DynamicThread::get_process()
{
  return *_parent;
}
    
Schedulable::state 
DynamicThread::get_state() const
{
  return _state;
}

Schedulable::state 
DynamicThread::set_state(state new_state)
{
  state old_state = _state;
  _state = new_state;

  return old_state;
}

vector<Request*> 
DynamicThread::get_requests()
{
  return vector<Request*>(_dynamic_requests.begin(), _dynamic_requests.end());
}

void 
DynamicThread::remove_request(Request* request)
{
  assert(request != NULL);
  
  vector<DynamicRequest*>::iterator it;
  
  it = std::find(_dynamic_requests.begin(), _dynamic_requests.end(), request);

  if(it != _dynamic_requests.end())
  {
    _dynamic_requests.erase(it);
    delete *it;
  }
  
}
    
void 
DynamicThread::add_request(DynamicRequest* request)
{
  assert(request != NULL);

  _dynamic_requests.push_back(request);
}

void
DynamicThread::serialize(SerializeVisitor& translator) const
{
  // TODO fill-in appropriate code
}
